# -*- coding: utf-8 -*-
# dreamtools/dtemng.py

"""
Module de Gestion des date
=============================

Liste de fonction pour utilisation des dates
pathfile : dreamtools/tools

Constantes globales
-------------------

Liste des jours de la semaine
I_MON, I_TUES, I_WED,I_THU, I_FRI, I_SAT, I_SUN = 1, 2, 3, 4,5,6,0

Fonctions
-------------

"""

import locale
import time
from datetime import datetime, timedelta, date

import pytz

I_MON, I_TUES, I_WED, I_THU, I_FRI, I_SAT, I_SUN = 1, 2, 3, 4, 5, 6, 0

locale.setlocale(locale.LC_TIME, 'fr_FR.utf8')

ISO_FORMAT = '%Y-%m-%dT%H:%M:%S'
FR_FORMAT = '%d/%m%/%Y'

FRM_ISO, FRM_TIMESTAMP = 'iso', 'ts'
tz_cay = pytz.timezone('America/Cayenne')
tz_fr = pytz.timezone('Europe/Paris')


def dte_dtime(dte):
    """convertit une date au format date en datetime à minuit"""
    if type(dte) is date:
        midnight = datetime.min.time()
        dte = datetime.combine(dte, midnight)
    return dte


def set_timezone(dt, tz=pytz.UTC):
    """ Applique la timezone indiquée à la date passée en parametre

    :param datetime dt: date
    :param timezone tz: timezone
    """
    return dt.replace(tzinfo=tz)


def datetime_from_utc_to_local(utc_datetime):
    """Convertie la date et heure donné (utc) en date local

    :param utc_datetime: datetime utc
    :return: date locale
    """
    return pytz.utc.localize(utc_datetime, is_dst=None).astimezone()


def datetime_from_local_to_utc(utc_datetime):
    """
    Convertie une date et heure local en heure utc

    :param datetime utc_datetime: datetime local
    :return: datatime utc
    """
    return utc_datetime.astimezone(pytz.utc)


def maintenant(utc=False, fm=None, tz=None):
    """ Date et heure de l'instant (Now)

        :param bool utc: Si True renvoie de l'heure UTC (GMT) ou l'heure local
        :param str fm: format date : fm == FRM_ISO (iso) | FRM_TIMESTAMP (ts)
        :param timezone tz: Timezone, None by def
        :rtype: datetime | string

        :Exemple:
            >>> maintenant ()
            datetime.datetime (2019, 06, 02, 17, 30, 43, 248622)
            >>> maintenant (True)
            '2019-06-02T17:30:43.248622'

    """
    d = datetime.utcnow()
    if not utc:
        d = pytz.utc.localize(d, is_dst=None).astimezone(tz)

    if fm == FRM_ISO:
        return d.isoformat()
    elif fm == FRM_TIMESTAMP:
        return dtets(d)
    else:
        return d


def utcnow_iso():
    """Date et heure actuelle utc au format iso
    :return: date utc
    """
    return maintenant(True, FRM_ISO)


def utcnow_ts():
    """Date et heure actuelle utc au format timestamp
    :return: timestamp
    """
    return maintenant(True, FRM_TIMESTAMP)


def dtestr(dte=None, fm='%Y-%m-%dT%H:%M:%S'):
    """ Convertit une date en chaine selon un format donnée

    :param datetime dte: date à convertir date du jour par défaut
    :param str fm: format désirée, defaults to  '%d/%m/%Y'
    :return: Renvoie un chaine correspondant au format date passé en parametre
    :rtype: str

    :Exemple:
        >>> d = maintenant ()
        >>> dtestr (d, '%d.%m.%Y')
        02.06.2019

    """

    if dte is None:
        dte = maintenant()

    return dte.strftime(fm)
  

def today(fm='%d/%m/%Y'):
    """ Renvoie la date du jour

        :param str fm: Format de la date attendu
        :rtype: str

        :Exemple:
            >>> today ()
            '02/06/2019'
            >>> today('%d.%m.%Y')
            '02.06.2019'

        """

    return dtestr(fm=fm)


def dte_dayed(dte=None, b=True):
    """ Positionne la date indiquée à minuit au matin ou au soir

    :param datetime dte: Date
    :param bool b: Date debut de jour (00:00:00.000) ou
        date de fin de journee date du jour + 1 (minuit) soit lendemin à 00

    :Example:
        >>> date_dayed()
        datetime.datetime(2020, 12, 19, 0, 0,datetime.datetime(2020, 12, 19, 0, 0)

    """

    if dte is None:
        dte = maintenant()

    dte = dte.replace(hour=0, minute=0, second=0, microsecond=0)

    return dte if b else dteadd(dte, 1)


def get_time(dte):
    """ Renvoie d'une date au format time
    :param datetime dte:
    :rtype: time
    """
    return dte.time()


def setdte(p_year, p_month, p_day):
    """Generationd'une date a partir des valeur numerique

    :param int p_year: année
    :param int p_month: mois
    :param int p_day: date jour
    :rtype: datetime
    """
    return date(p_year, p_month, p_day)


def tsdte(ts):
    """ Conversion timestamp - date

        :param ts: temps en milliseconde depuis 1970
        :return: date
    """
    return datetime.fromtimestamp(int(ts))  # => renvoie datetime


def tstring(ts, fm='%Y.%m.%d-%H:%M (%a)'):
    """
     Conversion timestamp - chaine(str)

     :param int ts: timestamp
     :param str fm: format attendu
     :rtype: str
    """
    return dtestr(tsdte(ts), fm)


def dtets(dte=None):
    """ Conversion date - timestamp

    :param date dte: date à convertir
    :return int: date en miliseconde (sans les ms)
    """
    if dte is None:
        dte = maintenant()
    elif type(dte) is date:
        pass

    return int(dte.timestamp())  # int => sans les ms


def strdte(dte, fm='%d-%m-%Y %H:%M:%S'):
    """ Conversion string - date

    :param str dte: date
    :param str fm: formayt, default '%d-%m-%Y %H:%M:%S': patterne, optional

    :return: Renvoie la date convertit ou None en cas d'invalidité (date non conform)
    :rtype: datetime

    :Exemple:
        >>> s = '24-O2-1976 16:45'
        >>> strdte (s, '%d-%m-%Y %H:%m')
        datetime.datetime(1976,02,24,16,45)

    """

    return datetime.strptime(dte, fm)


def isotodte(dte):
    """ Conversion str_iso - date

    Format ISO : YYYY-MM-DDTHH:MN
    :param dte:

    :return:
    """
    return strdte(dte, fm=ISO_FORMAT)


def paques(y):
    """Dates Pâques d'une année donnée

    * Lundi de paque : lundi suivant le dimanche de paque (La Pâque)
    * Jeudi de l'ascension : 3 jour après paques
    * pentecote : 49 jours après le lundi de paques

    :param int y: année de référence
    :rtype: list[date]
    """
    y = int(y)

    a = y // 100
    b = y % 100

    c = (3 * (a + 25)) // 4
    d = (3 * (a + 25)) % 4
    e = (8 * (a + 11)) // 25

    f = (5 * a + b) % 19
    g = (19 * f + c - e) % 30
    h = (f + 11 * g) // 319

    j = (60 * (5 - d) + b) // 4
    k = (60 * (5 - d) + b) % 4

    m = (2 * j - k - g + h) % 7
    n = (g - h + m + 114) // 31
    p = (g - h + m + 114) % 31

    jour = p + 1

    mois = n

    dte_paques = date(y, mois, jour)
    lundi_pak = dte_paques + timedelta(days=1)
    jeudi_ascension = dte_paques + timedelta(days=39)
    pentecote = lundi_pak + timedelta(days=49)

    return [dte_paques, lundi_pak, jeudi_ascension, pentecote]


def dteferies(y=None):
    """ Jour fériés pour une date donnée

    :param int y: Année de référence (optionnel), default : année en cours
    :return: un tableau de date de jours fériés

    :Exemple:
        >>> dteferies ()		#Jours fériés année en cours
        >>> dteferies (2018)	# jours fériés année 2018

    """

    y = maintenant().year if y is None else int(y)

    dt_feries = paques(y)
    #jour de lan, fete du travail, jour victoire, fete national; assomption, tousain, armistice, noël
    l_dte = ((1, 1), (5, 1), (5, 8), (7, 14), (8, 15), (10, 1), (11, 1), (12, 25))
    
    for month, day in l_dte:
        dt_feries.append(date(y, month, day))

    return dt_feries


def is_workday(dte):
    """ Determine si la date est un jour ouvré ou vaqué (week-end / fériés)

    :param dte: date à évaluer
    :return: renvoie le statut jour ouvré (true=ouvré)
    :rtype bool:
    """
    feries = dteferies(dte.year)
    return not (dte in feries or dte.weekday() in [I_SAT, I_SUN])


def dteadd(dte, nb, fm='d'):
    """ Ajoute un nombre de jours données à une date

    :param date dte: date de départ
    :param int nb: nombre de jour à additionner (valeur négative/positive)
    :param str fm: * days (default), * h (hours), * m (minutes)

    :rtype: datetime

    """
    if fm == 'h':
        return dte + timedelta(hours=nb)
    elif fm == 'm':
        return dte + timedelta(minutes=nb)
    else:
        return dte + timedelta(days=nb)


def timeadd(dte, nb):
    """  Ajoute un nombre d'heure données à une date

    :param date dte: date de départ
    :param int nb: nombre de jour à additionner (valeur négative/positive)

    :return: date de depat + nombre de jours
    """
    return dteadd(dte, nb, 'h')


def dte_add_workday(dte, nb):
    """ Ajoute un nombre de jours ouvrés donnés à une date

        :param datetime dte: date de référence
        :param int nb: nombre de jour à additionner (valeur négative/positive)
        :return: date de depat + nombre de jours
    """
    while nb > 0:
        dte = dteadd(dte, 1)
        if is_workday(dte):
            nb -= 1

    return dte


def dtediff(dtea, dteb):
    """ Calcul du nombre de jours entre deux dates

    :param datetime dtea: date à comparer
    :param datetime dteb: date à comparer
    :rtype: int
    """
    t = dteb - dtea
    return abs(t.days)


def dtecompare(dtea, dteb):
    """ Compare si dateb es supérieur à la datea

    :param datetime dtea: date à comparer
    :param datetime dteb: date à comparer
    :retur: 0 si les dates sont égale, -1 si la dateb est antérieur à datea 1 le c&s inverse
    :rtype: int
    """
    t = dteb - dtea

    if t.days == 0:
        return t.days
    elif t.days < 0:
        return -1
    else:
        return 1


def weeks_num(dte=None):
    """Renvoie le numéro de la date indiqué (now par deafut)"""
    if dte is None:
        dte = maintenant()

    return dte.isocalendar()[1]


def fullmonth(dte):
    """ Renvoie la date du jour au format MOIS YYYY

    :param datetime dte:
    :rtype: str
    """
    return dtestr(dte, "%B %Y")


def dterss(dte=None,short=False):
    """Dtate au format RSS """
    ctime = time if dte is None else time.mktime(dte.timetuple())
    return ctime.strftime('%a %d %b %Y') if short else ctime.strftime('%a, %d %b %Y %H:%M:%S %z')


def day_in_sec(dy, ml=False):
    """
    Convertion d'un nombre de jours en secondes ou milisecondes

    :param int dy: nombre de jours
    :param bool ml: en millisecondes si True sinon en secondes, dafault False
    :return: (milli) secondes
    """
    nb = int(dy)
    nb = nb * 24 * 60 * 60

    return nb * 1000 if ml else nb


def day_in_hour(dy):
    """ Convertion d'un nombre de jours en heure

    :param int dy: nombre de jours
    :rtype: int
    """
    nb = int(dy)
    return nb * 24

def dteTimeZoned (dte, tz=tz_cay):
    dte = dte.astimezone(tz)
    return dtestr(dte, '%Y%m%d%H%M00%z')


# -*- coding: utf-8 -*-
# features.py

"""
Module complémentaire
============================

pathfile : dreamtools/features.py

"""
import re
import requests

from urllib.parse import urljoin

from . import tools
from .logmng import CTracker


def test_http_link(url):
    """ Vérifie une url et renvoie l'url valide

    :param url: url à évaluer
    :rtype str:
    """

    def fn():
        s = tools.clean_allspace(url)

        if s[4] != 'http':
            s = f'http://{s}'

        ret = requests.head(s)

        if ret.status_code in [301, 302]:
            return test_http_link(ret.headers.get('Location'))

        return s if (ret.status_code == requests.codes.ok) else False

    return CTracker.fntracker(fn, 'URL Checker').data


def url_join(domaine, page):
    """
    Generation d'une url
    """

    return urljoin(domaine, page)



# -*- coding: utf-8 -*-
# dreamtools/logmng.py

import logging
import logging.config as log_config

from . import profiler
from . import tools
from .cfgmng import CFGBases


class CReponder:
    def __init__(self, status=200, data=None, msg=None):
        """ Constructeur de class

        :type status: int, default 200
        :param data: données renvoyée
        :param msg: message associée à la réponse

        """
        self.status = status
        self.data = data
        self.message = msg

    def __call__(self, status, data=None, msg=None):
        self.status = status
        self.data = data
        self.message = msg

    @property
    def ok(self):
        """ Renvoie le status de la réponse

        :rtype: bool
        """
        return 200 <= self._status < 300

    @property
    def response(self):
        """
        :return: {"message": self.message, "data": self.data, 'status_code': self._status}
        """
        return {"message": self.message, "data": self.data, 'status_code': self._status}

    @property
    def status(self):
        return self._status

    @property
    def data(self):
        return self._data

    @property
    def message(self):
        return self._msg

    @status.setter
    def status(self, v):
        self._status = v

    @message.setter
    def message(self, v):
        self._msg = v

    @data.setter
    def data(self, v):
        self._data = v


class CError(Exception, CReponder):
    """ Gestion des erreurs et traitement des exceptions personnalisées """

    def __init__(self, message, status, title='ERRCustom'):
        """
        :param int status: status code
        :param str message: message d'erreur
        :param str title: titre

        """
        Exception.__init__(self, message)
        CReponder.__init__(self, status, msg=message)

        self.title = title


class CTracker:
    tracker = None
    LOG_TRACKED = ''

    @classmethod
    def config(cls, mode='PRODUCTION'):
        """Initialisation du gestionnaire de log à partir de la configuration enregistré

        .. warning::
            La configuration doit être configurer dans le fichier <PROJECT_DIR>/cdg/.log.yml

        :Exemple:cls
            >>> CTracker.config()
            Configuration mode PRODUCTION
            >>> CTracker.config("DEBUG")
            Configuration mode debug

        """
        try:
            profiler.makedirs(profiler.path_build(tools.PROJECT_DIR, 'logs'))
            log_config.dictConfig(CFGBases.logs_cfg())
            cls.tracker = logging.getLogger(mode)
        except Exception as e:
            tools.print_err(e, ': ', 'Error in Logging Configuration. Using default configs')
            logging.basicConfig(level=logging.NOTSET)

    @classmethod
    def msg_tracking(cls, msg, title, log_level=logging.INFO, code=0):
        """ Tracking message

        :param str msg: message à ecrire dans logs
        :param str title: Titre ou référence associé au message
        :param int log_level: LOG LEVEL Niveau de l'alert (DEBUG | INFO | WARN | )
        :param int code: Code numérique

        """
        cls.tracker.log(log_level, msg, extra={'title': title, 'code': code})

    @classmethod
    def alert_tracking(cls, msg, title, code=0):
        """ Message d'alerte (WARNING)

        :param str msg: message à ecrire dans logs
        :param str title: Titre ou référence associé au message
        :param int code: Code numérique
        """

        cls.msg_tracking(msg, title, logging.WARNING, code)

    @classmethod
    def info_tracking(cls, msg, title, code=0):
        """ Message d'info (INFO)

        :param str msg: message à ecrire dans logs
        :param str title: Titre ou référence associé au message
        :param code: Code numérique
        """

        cls.msg_tracking(msg, title, logging.INFO, code)

    @classmethod
    def error_tracking(cls, msg, title, code=500):
        """ Message d'error (ERROR)

        :param str msg: message à ecrire dans logs
        :param str title: Titre ou référence associé au message
        :param int code: Code numérique
        """

        cls.msg_tracking(msg, title, logging.ERROR, code)

    @classmethod
    def critical_tracking(cls, msg, title, code=0):
        """ Message dcritique (CRITIQUE)

        :param str msg: message à ecrire dans logs
        :param str title: Titre ou référence associé au message
        :param int code: Code numérique
        """
        cls.msg_tracking(msg, title, logging.CRITICAL, code)

    @classmethod
    def flag(cls, trace):
        """ Permet de pointer la dernier action

        :param str trace: Action à enregistrer
        """
        cls.LOG_TRACKED = trace

    @classmethod
    def exception_tracking(cls, ex, title, status=500):
        """
        Récupération et traitement des exceptions

        :param ex: Exception
        :param str title: Information
        """

        try:
            if cls.LOG_TRACKED != '':
                cls.msg_tracking(cls.LOG_TRACKED, title, logging.INFO)
                cls.LOG_TRACKED = ''
        except :
            pass

        try:
            if isinstance(ex, CError):
                cls.error_tracking(ex.message, ex.title, code=ex.status or status)
                return ex
            else:
                cls.error_tracking(ex.__str__(), title)
                return CError(ex.__str__(), status, title)
        except Exception as sex:
            tools.print_err('*****************  Erreur systeme *****************: \n', sex)
            tools.print_err('** Erreur interceptée : ', ex)

            return CError(ex, status=status)

    @classmethod
    def fntracker(cls, fn, action, *args, **kwargs):
        """ Execution "securisé" d'une fonction avec gestions des erreurs
        
        :param fn: fonction a executer
        :param action: Titre de l'execution pour tracabilité
        :param args: argument de la fonction
        :param kwargs: parametres supplementaire (status par defaut en cas de reussite)
        :rtype: CReponder

        :Exemple:
            >>> from dreamtools.logmng import CTracker
            >>> def fn(arg)::
            >>>     return int(arg)
            >>> r = CTracker.fntracker(fn, 'Test de convertion int', 'j')
            >>> r.response
            {'message': "invalid literal for int() with base 10: 'j'", 'data': None, 'status_code': 500}
            >>> r = CTracker.fntracker(fn, 'Test de convertion int', '589321')
            >>> r.response
            {'message': None, 'data': 589321, 'status_code': 200}
        """

        try:
            cls.flag('{}'.format(action))
            status = kwargs.get('status', 200)

            if kwargs.get('status'): del kwargs['status']
            r = fn(*args, **kwargs)

            return r if isinstance(r, CReponder) else CReponder(status, r)

        except Exception as ex:
            return cls.exception_tracking(ex, action)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

import dreamtools

cfg = ["cfg/*.yml", '.env']

with open('README.md', encoding='utf-8') as readme:
    long_description = readme.read()


#setup------------------------------
setup(

    name='dreamtools-dreamgeeker',
    version=dreamtools.__version__,
    packages=find_packages(),
    author="dreamgeeker",
    author_email="dreamgeeker@couleurwest-it.com",
    description="outils de developpement de base",
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=['wheel', 'setuptools', 'pyaml', 'requests', 'cerberus', 'pillow', 'pytz',
                      "urllib3", "cerberus", "pillow", 'python-dotenv', 'verify_email', 'beautifulsoup4'],

    include_package_data=True,
    python_requires='>=3.8',
    url='https://github.com/couleurwest/dreamgeeker-tools',
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: French",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.8",
        "Topic :: Software Development",
    ],
    package_data={'dreamtools': cfg},
    entry_points={
        'console_scripts': [
            'tools-installer = scripts.__main__:setproject'
        ]
    }
)




class CConfig:

    def __init__(self, project_name=None, mode="PRODUCTION"):
        """
        Configuration des parametres d'accès relatif au projet en cours
        ==============================================================

        Parametres
        -----------
        :param str app: Nom de l'application
        :param str mode: PRODUCTION | DEVELOPMENT

        """
        import os
        from . import tools
        from .logmng import CTracker
        from dreamtools import profiler

        from dotenv import load_dotenv

        tools.APP_NAME = os.getenv('PROJECT_NAME', project_name)
        tools.PROJECT_DIR = profiler.dirproject()
        tools.APP_DIR = profiler.path_build(tools.PROJECT_DIR, tools.APP_NAME)
        tools.TMP_DIR = profiler.path_build(tools.PROJECT_DIR, 'tmp')

        load_dotenv(profiler.path_build(tools.PROJECT_DIR, '.env'))

        CTracker.config(os.getenv('MODE', mode))
